//   ______
// _| R250 |__________________________________________________________________
//	the r250 uniform random number algorithm
//
//	Kirkpatrick, S., and E. Stoll, 1981; "A Very Fast
//	Shift-Register Sequence Random Number Generator",
//	Journal of Computational Physics, V.40
//
//	see also: W.L. Maier, DDJ May 1991
//

#include "lcg.h"

#include "r250.h"

R250::R250(uint32 seed) { Seed(seed); }

void R250::Seed (uint32 seed)
{
// we use an LCG to seed this class :-)
	LCG lcg (seed);
	index = 0;

	int j;

	for (j = 0; j < 250; j++)	// fill r250 buffer with bit values 
		buffer[j] = lcg.lRand();
	
	for (j = 0; j < 250; j++)	// set some MSBs to 1 
		if ( lcg.lRand() > 0x20000000L  )
			buffer[j] |= 0x40000000L ;

	uint32 msb = 0x40000000L ;		// turn on diagonal bit 
	uint32 mask = 0x7fffffffL ;	// turn off the leftmost bits 

	for (j = 0; j < 31 ; j++)
	{
		int k = 7 * j + 3;	// select a word to operate on 
		buffer[k] &= mask;	// turn off bits left of the diagonal 
		buffer[k] |= msb;	// turn on the diagonal bit 
		mask >>= 1;
		msb  >>= 1;
	}

}

//--[returns a random unsigned integer]---------------------------------------
uint32
R250::lRand ()
{
	long	j;

	if ( index >= 147 )
		j = index - 147;	// wrap pointer around 
	else
		j = index + 103;

	uint32 new_rand = buffer[ index ] ^ buffer[ j ];
	buffer[ index ] = new_rand;

	if ( index >= 249 )	// increment pointer for next time 
		index = 0;
	else
		index++;

	return new_rand;

}


//--[returns a random double in range 0..1]-----------------------------------
double
R250::dRand ()
{
	uint32 new_rand;

	new_rand = lRand ();
#ifdef USE_DIVISION
	return (double)new_rand / 0x7fffffffL ;
#else
#define RECIP (1.0 / 2147483647.0)
	return (double)new_rand * RECIP ;
#endif

}

//--[set Info static member]--
static RNGInfoItem r250items[] =
{
	{ "Class", "R250"},
	{ "Description", "the r250 uniform random number algorithm" },
	{ "Authors", "Kirkpatrick, S., and E. Stoll, 1981" },
	{ "Article", "A Very Fast Shift-Register Sequence Random Number Generator"},
	{ "Journal", "Journal of Computational Physics, V.40" },
	{ "See-Also", "W.L. Maier, DDJ May 1991" }
} ;

#define NITEMS (sizeof(r250items)/sizeof(r250items[0]))

static RNGInfo r250info[] = { NITEMS, r250items } ;

RNGInfo *R250::info = r250info;

