#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <limits.h>
#include <OS.h>

#include "random.h"

enum Algo
{
	ALGO_LCG,
	ALGO_R250,
	ALGO_MT,
	ALGO_ISAAC,
	ALGO_MOTHER,
	N_ALGO
} ;

static void usage()
{
	fprintf(stderr, "Usage: testrand [-q] [-n iterations] [-s seed]");
	fprintf(stderr, " [-L|-R|-T|-I|-M] \n");
	fprintf(stderr, " -q = quiet (no copyright info)\n");
	fprintf(stderr, " -qq = very quiet (no class name)\n");
	fprintf(stderr, " -L = use Linear Congruential (default)\n");
	fprintf(stderr, " -R = use R250\n");
	fprintf(stderr, " -T = use Mersenne Twister\n");
	fprintf(stderr, " -I = use ISAAC\n");
	fprintf(stderr, " -M = use Mother\n");
	fprintf(stderr, " (default number of iterations is 10000)\n");
}

static void
testrand (
	Algo algo,
	int n,
	int seed,
	int quiet
)
{
// pointer to base class
	RNG *rng;

	switch(algo)
	{
	default:
	case ALGO_LCG:
		rng = new LCG (seed);
		break;
	case ALGO_R250:
		rng = new R250 (seed);
		break;
	case ALGO_MT:
		rng = new Twister (seed);
		break;
	case ALGO_ISAAC:
		rng = new ISAAC (seed);
		break;
	case ALGO_MOTHER:
		rng = new Mother (seed);
		break;
	}

	if(quiet == 0)
		rng->Info()->PrintToStream();

// just print the first item's value (should be class name)
	else if(quiet == 1)
		printf("[%s]\n", rng->Info()->item->value);

	double tot = 0.0;
	uint32 minl = LONG_MAX, maxl = 0;

	bigtime_t start = system_time();
	for(int i=0; i<n; i++)
	{
		uint32 l = rng->lRand();
		double d = rng->dRand();

		tot += d;

		if(l > maxl)
			maxl = l;
		if(l < minl)
			minl = l;
	}
	bigtime_t end = system_time();

	printf("Completed %d iterations (%d integer + %d floating-point)\n",
		n*2, n, n
	);
	printf(" in %Ld microseconds = %g iters/sec\n", end-start, 500000.0*n/(end-start));
	printf("The average real number was %g\n", tot/n);
	printf("The min and max integers were %u,%u\n", minl, maxl);
	printf("\n");

	delete rng;
}

// we have to fudge this to get getopt to compile! 
extern "C" {
extern int getopt (int argc, char *const *argv, const char *shortopts);

extern char *optarg;
extern int optind;
}

static int args (
	int argc,
	char **argv,
	int *nitersP,
	int *seedP,
	Algo *algoP,
	int *quietP
)
{
	int opt;
	int erropt = 0;

	while( (opt = getopt(argc, argv, "RLTIMn:s:q")) != EOF)
	{
		switch(opt)
		{
		case 'q':
			*quietP += 1;
			break;
		case 'R':
			*algoP = ALGO_R250;
			break;
		case 'L':
			*algoP = ALGO_LCG;
			break;
		case 'T':
			*algoP = ALGO_MT;
			break;
		case 'I':
			*algoP = ALGO_ISAAC;
			break;
		case 'M':
			*algoP = ALGO_MOTHER;
			break;
		case 'n':
			*nitersP = atoi(optarg);
			break;
		case 's':
			*seedP = atoi(optarg);
			break;
		default:
			erropt++;
			break;
		}
	}

	return erropt;
}
int main (int argc, char **argv)
{
	int erropt = 0;
	Algo algo = ALGO_LCG;
	int niters = 10000;
	int seed = 0xDEC0DED;
	int quiet = 0;

	erropt = args(argc, argv, &niters, &seed, &algo, &quiet);

	if(erropt)
		usage();
	else
		testrand(algo, niters, seed, quiet);

	return erropt ? EINVAL : 0;
}
